This document describes some of the major changes between the 2.0 and
2.2 versions of the Apache HTTP Server. For new features since version
1.3, see the

2.0 new features

document.

Core Enhancements {#core}
=================

Authn/Authz

The bundled authentication and authorization modules have been
refactored. The new mod\_authn\_alias(already removed from 2.3/2.4)
module can greatly simplify certain authentication configurations. See

module name changes

, and

the developer changes

for more information about how these changes affects users and module
writers.

Caching

mod\_cache

,

mod\_cache\_disk

, and mod\_mem\_cache(already removed from 2.3/2.4) have undergone a lot
of changes, and are now considered production-quality.

htcacheclean

has been introduced to clean up

mod\_cache\_disk

setups.

Configuration

The default configuration layout has been simplified and modularised.
Configuration snippets which can be used to enable commonly-used
features are now bundled with Apache, and can be easily added to the
main server config.

Graceful stop

The

prefork

,

worker

and

event

MPMs now allow

httpd

to be shutdown gracefully via the

graceful-stop

signal. The

GracefulShutdownTimeout

directive has been added to specify an optional timeout, after which

httpd

will terminate regardless of the status of any requests being served.

Proxying

The new

mod\_proxy\_balancer

module provides load balancing services for

mod\_proxy

. The new

mod\_proxy\_ajp

module adds support for the

Apache JServ Protocol version 1.3

used by

Apache Tomcat

.

Regular Expression Library Updated

Version 5.0 of the

Perl Compatible Regular Expression Library

(PCRE) is now included.

httpd

can be configured to use a system installation of PCRE by passing the

\--with-pcre

flag to configure.

Smart Filtering

mod\_filter

introduces dynamic configuration to the output filter chain. It enables
filters to be conditionally inserted, based on any Request or Response
header or environment variable, and dispenses with the more problematic
dependencies and ordering problems in the 2.0 architecture.

Large File Support

httpd

is now built with support for files larger than 2GB on modern 32-bit
Unix systems. Support for handling \>2GB request bodies has also been
added.

Event MPM

The

event

MPM uses a separate thread to handle Keep Alive requests and accepting
connections. Keep Alive requests have traditionally required httpd to
dedicate a worker to handle it. This dedicated worker could not be used
again until the Keep Alive timeout was reached.

SQL Database Support

mod\_dbd

, together with the

apr\_dbd

framework, brings direct SQL support to modules that need it. Supports
connection pooling in threaded MPMs.

Module Enhancements {#module}
===================

Authn/Authz

Modules in the aaa directory have been renamed and offer better support
for digest authentication. For example,

mod\_auth

is now split into

mod\_auth\_basic

and

mod\_authn\_file

;

mod\_auth\_dbm

is now called

mod\_authn\_dbm

;

mod\_access

has been renamed

mod\_authz\_host

. There is also a new mod\_authn\_alias(already removed from 2.3/2.4)
module for simplifying certain authentication configurations.

mod\_authnz\_ldap

This module is a port of the 2.0

mod\_auth\_ldap

module to the 2.2

Authn/Authz

framework. New features include using LDAP attribute values and
complicated search filters in the

Require

directive.

mod\_authz\_owner

A new module that authorizes access to files based on the owner of the
file on the file system

mod\_version

A new module that allows configuration blocks to be enabled based on the
version number of the running server.

mod\_info

Added a new

?config

argument which will show the configuration directives as parsed by
Apache, including their file name and line number. The module also shows
the order of all request hooks and additional build information, similar
to

httpd -V

.

mod\_ssl

Added a support for

RFC 2817

, which allows connections to upgrade from clear text to TLS encryption.

mod\_imagemap

mod\_imap

has been renamed to

mod\_imagemap

to avoid user confusion.

Program Enhancements {#programs}
====================

httpd

A new command line option

-M

has been added that lists all modules that are loaded based on the
current configuration. Unlike the

-l

option, this list includes DSOs loaded via

mod\_so

.

httxt2dbm

A new program used to generate dbm files from text input, for use in

RewriteMap

with the

dbm

map type.

Module Developer Changes {#developer}
========================

APR

1.0 API

Apache 2.2 uses the APR 1.0 API. All deprecated functions and symbols
have been removed from

APR

and

APR-Util

. For details, see the

APR Website

.

Authn/Authz

The bundled authentication and authorization modules have been renamed
along the following lines:

mod\_auth\_\*

-\> Modules that implement an HTTP authentication mechanism

mod\_authn\_\*

-\> Modules that provide a backend authentication provider

mod\_authz\_\*

-\> Modules that implement authorization (or access)

mod\_authnz\_\*

-\> Module that implements both authentication & authorization

There is a new authentication backend provider scheme which greatly
eases the construction of new authentication backends.

Connection Error Logging

A new function,

ap\_log\_cerror

has been added to log errors that occur with the client\'s connection.
When logged, the message includes the client IP address.

Test Configuration Hook Added

A new hook,

test\_config

has been added to aid modules that want to execute special code only
when the user passes

-t

to

httpd

.

Set Threaded MPM\'s Stacksize

A new directive,

ThreadStackSize

has been added to set the stack size on all threaded MPMs. This is
required for some third-party modules on platforms with small default
thread stack size.

Protocol handling for output filters

In the past, every filter has been responsible for ensuring that it
generates the correct response headers where it affects them. Filters
can now delegate common protocol management to

mod\_filter

, using the

ap\_register\_output\_filter\_protocol

or

ap\_filter\_protocol

calls.

Monitor hook added

Monitor hook enables modules to run regular/scheduled jobs in the parent
(root) process.

Regular expression API changes

The

pcreposix.h

header is no longer available; it is replaced by the new

ap\_regex.h

header. The POSIX.2

regex.h

implementation exposed by the old header is now available under the

ap\_

namespace from

ap\_regex.h

. Calls to

regcomp

,

regexec

and so on can be replaced by calls to

ap\_regcomp

,

ap\_regexec

.

DBD Framework (SQL Database API)

With Apache 1.x and 2.0, modules requiring an SQL backend had to take
responsibility for managing it themselves. Apart from reinventing the
wheel, this can be very inefficient, for example when several modules
each maintain their own connections.

Apache 2.1 and later provides the

ap\_dbd

API for managing database connections (including optimised strategies
for threaded and unthreaded MPMs), while APR 1.2 and later provides the

apr\_dbd

API for interacting with the database.

New modules SHOULD now use these APIs for all SQL database operations.
Existing applications SHOULD be upgraded to use it where feasible,
either transparently or as a recommended option to their users.
