Rewrite

This document supplements the

mod\_rewrite

reference documentation

. It describes how you can use

mod\_rewrite

to create dynamically configured virtual hosts.

> **Note**
>
> mod\_rewrite is usually not the best way to configure virtual hosts.
> You should first consider the
>
> alternatives
>
> before resorting to mod\_rewrite. See also the \"
>
> how to avoid mod\_rewrite
>
> document.

Module documentation

mod\_rewrite introduction

Redirection and remapping

Controlling access

Proxying

RewriteMap

Advanced techniques

When not to use mod\_rewrite

Virtual Hosts For Arbitrary Hostnames {#per-hostname}
=====================================

Description:

We want to automatically create a virtual host for every hostname which
resolves in our domain, without having to create new VirtualHost
sections.

In this recipe, we assume that we\'ll be using the hostname

SITE

.example.com

for each user, and serve their content out of

/home/

SITE

/www

. However, we want

www.example.com

to be omitted from this mapping.

Solution:

RewriteEngine on RewriteMap lowercase int:tolower RewriteCond
%{HTTP\_HOST} !\^www\\. RewriteCond \${lowercase:%{

HTTP\_HOST

}} \^

(\[\^.\]+)

\\.example\\.com\$ RewriteRule \^(.\*) /home/

\%1

/www\$1

Discussion

> **Note**
>
> You will need to take care of the DNS resolution - Apache does not
> handle name resolution. You\'ll need either to create CNAME records
> for each hostname, or a DNS wildcard record. Creating DNS records is
> beyond the scope of this document.

The internal

tolower

RewriteMap directive is used to ensure that the hostnames being used are
all lowercase, so that there is no ambiguity in the directory structure
which must be created.

Parentheses used in a

RewriteCond

are captured into the backreferences

\%1

,

\%2

, etc, while parentheses used in

RewriteRule

are captured into the backreferences

\$1

,

\$2

, etc.

The first

RewriteCond

checks to see if the hostname starts with

www.

, and if it does, the rewriting is skipped.

As with many techniques discussed in this document, mod\_rewrite really
isn\'t the best way to accomplish this task. You should, instead,
consider using

mod\_vhost\_alias

instead, as it will much more gracefully handle anything beyond serving
static files, such as any dynamic content, and Alias resolution.

Dynamic Virtual Hosts Using mod\_rewrite {#simple.rewrite}
========================================

This extract from

httpd.conf

does the same thing as

the first example

. The first half is very similar to the corresponding part above, except
for some changes, required for backward compatibility and to make the

mod\_rewrite

part work properly; the second half configures

mod\_rewrite

to do the actual work.

Because

mod\_rewrite

runs before other URI translation modules (e.g.,

mod\_alias

),

mod\_rewrite

must be told to explicitly ignore any URLs that would have been handled
by those modules. And, because these rules would otherwise bypass any

ScriptAlias

directives, we must have

mod\_rewrite

explicitly enact those mappings.

\# get the server name from the Host: header UseCanonicalName Off \#
splittable logs LogFormat \"%{Host}i %h %l %u %t \\\"%r\\\" %s %b\"
vcommon CustomLog \"logs/access\_log\" vcommon \<Directory
\"/www/hosts\"\> \# ExecCGI is needed here because we can\'t force \#
CGI execution in the way that ScriptAlias does Options FollowSymLinks
ExecCGI \</Directory\> RewriteEngine On \# a ServerName derived from a
Host: header may be any case at all RewriteMap lowercase \"int:tolower\"
\#\# deal with normal documents first: \# allow Alias /icons/ to work -
repeat for other aliases RewriteCond \"%{REQUEST\_URI}\" \"!\^/icons/\"
\# allow CGIs to work RewriteCond \"%{REQUEST\_URI}\" \"!\^/cgi-bin/\"
\# do the magic RewriteRule \"\^/(.\*)\$\"
\"/www/hosts/\${lowercase:%{SERVER\_NAME}}/docs/\$1\" \#\# and now deal
with CGIs - we have to force a handler RewriteCond \"%{REQUEST\_URI}\"
\"\^/cgi-bin/\" RewriteRule \"\^/(.\*)\$\"
\"/www/hosts/\${lowercase:%{SERVER\_NAME}}/cgi-bin/\$1\"
\[H=cgi-script\]

Using a Separate Virtual Host Configuration File {#xtra-conf}
================================================

This arrangement uses more advanced

mod\_rewrite

features to work out the translation from virtual host to document root,
from a separate configuration file. This provides more flexibility, but
requires more complicated configuration.

The

vhost.map

file should look something like this:

customer-1.example.com /www/customers/1

customer-2.example.com /www/customers/2

\# \...

customer-N.example.com /www/customers/N

The

httpd.conf

should contain the following:

RewriteEngine on RewriteMap lowercase \"int:tolower\" \# define the map
file RewriteMap vhost \"txt:/www/conf/vhost.map\" \# deal with aliases
as above RewriteCond \"%{REQUEST\_URI}\" \"!\^/icons/\" RewriteCond
\"%{REQUEST\_URI}\" \"!\^/cgi-bin/\" RewriteCond
\"\${lowercase:%{SERVER\_NAME}}\" \"\^(.+)\$\" \# this does the
file-based remap RewriteCond \"\${vhost:%1}\" \"\^(/.\*)\$\" RewriteRule
\"\^/(.\*)\$\" \"%1/docs/\$1\" RewriteCond \"%{REQUEST\_URI}\"
\"\^/cgi-bin/\" RewriteCond \"\${lowercase:%{SERVER\_NAME}}\"
\"\^(.+)\$\" RewriteCond \"\${vhost:%1}\" \"\^(/.\*)\$\" RewriteRule
\"\^/cgi-bin/(.\*)\$\" \"%1/cgi-bin/\$1\" \[H=cgi-script\]
